function attachSecureInput() {
    var createButton = function (btn, id) {
        var left   = btn.offset().left,
            top    = btn.offset().top,
            width  = btn.outerWidth(),
            height = btn.outerHeight();

        return secure.promptButton(id, id, left, top, width, height);
    };

    secure.promptClear();
    $(".menu-item,.close").each(function () {
        try {
            createButton($(this), this.id);
        } catch (e) {
            console.log("failed to create secure input button: " + e.message);
        }
    });
    secure.promptGesture("abc", 0, 100, 100);
    var res = secure.promptEvent(0x7FFFFFFF);
    if (res.status) {
        console.log("secure.promptEvent failed: " + JSON.stringify(res));
    }
}

secure.onSecureInput.connect(function (data) {
    secure.promptEvent(-1);

    for (var i in data) {
        $(".menu-item").removeClass("selected");
        var id = "#" + data[i];
        var el = $(id);

        el.addClass("selected");
        el.click();

        return;
    }
});

var createMenuItem = function (menuItem, id) {
    var el = document.createElement("div");
    el.className = "menu-item";
    el.id = id;

    // icon
    var div = $(el);
    if (menuItem.icon) {
        div.append("<div class='icon'><img src='" + menuItem.icon + "'/></div>");
    }

    // text
    if (menuItem.title) {
        div.append("<div class='title'>" + menuItem.title + "</div>");
    }

    // on click
    div.click(function () {
        var path = menuItem.target.replace(/^html\//, settings.appName + "/");
        var rc = page.showPage("/cgi-bin/page/show?path=" + encodeURI(path));
        if (rc !== 0) {
            console.log("cannot show page: " + path);
        }
    });

    document.getElementById('menu').appendChild(el);
};

var menuHandler = function (menuItems, optionalKeys) {
    $("#menu").empty();

    var id = 1;
    for (var i in menuItems) {
        createMenuItem(menuItems[i], id++);
    }
    createNextButton();
    attachSecureInput();
    monitorOptionalKeys(optionalKeys);
};

// *next* button
var createNextButton = function () {
    if (descriptor.pageCount() < 2) {
        return;
    }

    var el = document.createElement("div");
    el.className = "menu-item";
    el.id = "btn-next";

    // icon
    var div = $(el);
    div.append("<div class='icon'><img src='assets/images/next.png'/></div>");

    // text
    div.append("<div class='title'>Next</div>");

    // on click
    div.click(function () {
        descriptor.next(menuHandler);
    });

    document.getElementById('menu').appendChild(el);
};

var scrollLeft = function () {
    try {
        descriptor.next(menuHandler);
    } catch (e)  {
        console.log('error: ' + e.message);
    }
};

var scrollRight = function () {
    try {
        descriptor.previous(menuHandler);
    } catch (e) {
        console.log('error: ' + e.message);
    }
};

secure.onGesture.connect(function (data) {
    try {
        if (!data) {
            return;
        }

        if (data.type == 'end') {
            if (data.x > 0) {
                scrollLeft();
            } else {
                scrollRight();
            }
        }
    } catch (e) {
        console.log('onGesture err: ' + e.message);
    }
});

function loadMenu() {
    var path = '/files/' + settings.appName + '/descriptor.json';
    descriptor.load(path, menuHandler);
}

var monitorOptionalKeys = function (optionalKeys) {
    if (monitorOptionalKeys.inProgress) {
        return;
    }

    if (optionalKeys.length === 0) {
        return;
    }

    for (var i = 0; i < optionalKeys.length; i++) {
        registry.subscribe(optionalKeys[i]);
    }

    registry.valueChanged.connect(function (key, value) {
        if (optionalKeys.indexOf(key) !== -1) {
            loadMenu();
        }
    });

    monitorOptionalKeys.inProgress = true;
};

loadMenu();

// beeper
try {
    secure.toggleBeeper(true);
    secure.setBeeperParams(2000, 500, 30);
} catch (e) {
    console.log("failed to set beeper params: " + e.message);
}

